@@ -43,11 +43,11 @@
 #define IMAGE_ALPHA_NORMAL		1.0
 #define IMAGE_ALPHA_HIGHLIGHT	0.65
 
-// #define _DEBUG
+#define _DEBUG
 #ifdef _DEBUG
-// redefine // g_debug() to g_message()
+// redefine g_debug() to g_message()
 // TODO: find out how to get debug logs from hildon-desktop
-#define // g_debug g_message
+#define g_debug g_message
 #define USER_AGENT_VERSION_STRING (debug version)
 #else
 #define USER_AGENT_VERSION_STRING (some version number)
@@ -121,6 +121,8 @@ static void gtk_sleep (unsigned int ms)
 static dooris_state_t
 fetch_current_state (CCCHHDoorIsDesktopWidget *self)
 {
+	static GStaticMutex local_fetch_mutex = G_STATIC_MUTEX_INIT;
+
 	gulong ts;
 	dooris_state_t state;
 	char *line, *tokctx;
@@ -132,40 +134,49 @@ fetch_current_state (CCCHHDoorIsDesktopWidget *self)
 	char buffer[MAX_HTTP_SIZE+1];
 	memset(buffer, 0, sizeof(buffer));
 
-	// g_debug("ccchh-dooris-widget: fetch_current_state()");
-	// g_debug("ccchh-dooris-widget: fetching state from URL: %s", self->url);
+	g_debug("ccchh-dooris-widget: fetch_current_state()");
+
+	if (!g_static_mutex_trylock(&local_fetch_mutex)) {
+		g_warning("ccchh-dooris-widget: BUG: local fetch() lock set, not updating");
+		return DOORIS_FNORD;
+	}
+
+	g_debug("ccchh-dooris-widget: fetching state from URL: %s", self->url);
 
 // why bother? we check error on open
 /*
 GnomeVFSURI *uri_struct = gnome_vfs_uri_new(self->url);
 if (!gnome_vfs_uri_exists(uri_struct)) {
 	g_warning("ccchh-dooris-widget: URL does not exist");
+	g_static_mutex_unlock(&local_fetch_mutex);
 	return DOORIS_FNORD;
 }
 gnome_vfs_uri_unref(uri_struct);
 */
 
-	// g_debug("ccchh-dooris-widget: gnome_vfs_open()");
+	g_debug("ccchh-dooris-widget: gnome_vfs_open()");
 	gvfs_errno = gnome_vfs_open(&gvfs_handle, self->url, GNOME_VFS_OPEN_READ);
 
 	if (gvfs_errno != GNOME_VFS_OK) {
 		g_warning("ccchh-dooris-widget: gnome_vfs_open: %s (%d)\n", gnome_vfs_result_to_string(gvfs_errno), gvfs_errno);
+		g_static_mutex_unlock(&local_fetch_mutex);
 		return DOORIS_FNORD;
 	}
 
-	// g_debug("ccchh-dooris-widget: gnome_vfs_read()");
+	g_debug("ccchh-dooris-widget: gnome_vfs_read()");
 	gvfs_errno = gnome_vfs_read(gvfs_handle, buffer, MAX_HTTP_SIZE, &bytes_read);
 
 	if (gvfs_errno != GNOME_VFS_OK) {
 		g_warning("ccchh-dooris-widget: gnome_vfs_read: %s (%d)\n", gnome_vfs_result_to_string(gvfs_errno), gvfs_errno);
+		g_static_mutex_unlock(&local_fetch_mutex);
 		return DOORIS_FNORD;
 	}
 
-	// g_debug("ccchh-dooris-widget: read %lld bytes", bytes_read);
+	g_debug("ccchh-dooris-widget: read %lld bytes", bytes_read);
 // rest of buffer already zeroed by memset()
 //	buffer[bytes_read] = '\0';
 
-	// g_debug("ccchh-dooris-widget: gnome_vfs_close()");
+	g_debug("ccchh-dooris-widget: gnome_vfs_close()");
 	gnome_vfs_close(gvfs_handle);
  
 	self->last_change_ts = 0L;
@@ -176,18 +187,20 @@ gnome_vfs_uri_unref(uri_struct);
 
 	if (!(line = strtok_r(buffer, "\n", &tokctx))) {
 		g_warning("ccchh-dooris-widget: first line of response not found");
+		g_static_mutex_unlock(&local_fetch_mutex);
 		return DOORIS_FNORD;
 	}
 
 	if (!strcmp(line, "locked")) {
 		state = DOORIS_CLOSED;
-		// g_debug("ccchh-dooris-widget: current state: closed");
+		g_debug("ccchh-dooris-widget: current state: closed");
 	} else if (!strcmp(line, "unlocked")) {
 		state = DOORIS_OPEN;
-		// g_debug("ccchh-dooris-widget: current state: open");
+		g_debug("ccchh-dooris-widget: current state: open");
 	}
 	else {
 		g_warning("ccchh-dooris-widget: could not parse response");
+		g_static_mutex_unlock(&local_fetch_mutex);
 		return DOORIS_FNORD;
 	}
 
@@ -199,7 +212,7 @@ gnome_vfs_uri_unref(uri_struct);
 		if (ts && !errno) {
 			self->last_report_ts = ts;
 		}
-		// g_debug("ccchh-dooris-widget: last_report_ts: %lu, errno: %i\n", ts, errno);
+		g_debug("ccchh-dooris-widget: last_report_ts: %lu, errno: %i\n", ts, errno);
 
 		// third line of body is last state change timestamp
 
@@ -209,11 +222,13 @@ gnome_vfs_uri_unref(uri_struct);
 			if (ts && !errno) {
 				self->last_change_ts = ts;
 			}
-			// g_debug("ccchh-dooris-widget: last_change_ts: %lu, errno: %i\n", ts, errno);
+			g_debug("ccchh-dooris-widget: last_change_ts: %lu, errno: %i\n", ts, errno);
 		}
 	}
 
- 	return state;
+	g_static_mutex_unlock(&local_fetch_mutex);
+
+	return state;
 }
 
 // stop update timer
@@ -224,7 +239,7 @@ stop_update_timer(CCCHHDoorIsDesktopWidget *self)
 {
 	// stop timer if it was running
 	if (self->timeout_handler) {
-		// g_debug("ccchh-dooris-widget: stopping update timer");
+		g_debug("ccchh-dooris-widget: stopping update timer");
 		g_source_remove (self->timeout_handler);
 		self->timeout_handler = 0;
 		return TRUE;
@@ -240,7 +255,7 @@ static inline gboolean
 start_update_timer(CCCHHDoorIsDesktopWidget *self)
 {
 	if (!self->timeout_handler) {
-		// g_debug("ccchh-dooris-widget: starting update timer");
+		g_debug("ccchh-dooris-widget: starting update timer");
 		if (self->update < UPDATE_INTERVAL_MIN) {
 			g_warning("ccchh-dooris-widget: self->update < UPDATE_INTERVAL_MIN, maybe a bug?");
 			self->update = UPDATE_INTERVAL_MIN;
@@ -269,7 +284,7 @@ install_connection_handler(CCCHHDoorIsDesktopWidget *self)
 			G_CALLBACK (ccchh_dooris_desktop_widget_connection_event), self);
 		/* Set automatic events */
 		g_object_set (self->connection, "automatic-connection-events", TRUE, NULL);
-		// g_debug("ccchh-dooris-widget: connection monitoring enabled");
+		g_debug("ccchh-dooris-widget: connection monitoring enabled");
 		return TRUE;
 	}
 
@@ -286,7 +301,7 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 		g_object_set (self->connection, "automatic-connection-events", FALSE, NULL);
 		g_signal_handler_disconnect (self->connection, self->connection_handler);
 		self->connection_handler = 0;
-		// g_debug("ccchh-dooris-widget: connection monitoring disabled");
+		g_debug("ccchh-dooris-widget: connection monitoring disabled");
 		return TRUE;
 	}
 
@@ -302,10 +317,18 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 static gboolean
 update (CCCHHDoorIsDesktopWidget *self)
 {
-	// g_debug("ccchh-dooris-widget: update()");
+	static GStaticMutex local_update_mutex = G_STATIC_MUTEX_INIT;
+
+	g_debug("ccchh-dooris-widget: update()");
+
+	if (!g_static_mutex_trylock(&local_update_mutex)) {
+		g_warning("ccchh-dooris-widget: BUG: local update() lock set, not updating");
+		return FALSE;
+	}
 
 	if (!self->connected) {
-		// g_debug("ccchh-dooris-widget: not connected, not fetching new state");
+		g_debug("ccchh-dooris-widget: not connected, not fetching new state");
+		g_static_mutex_unlock(&local_update_mutex);
 		return FALSE;
 	}
 
@@ -313,7 +336,8 @@ update (CCCHHDoorIsDesktopWidget *self)
 	self->current_state = fetch_current_state(self);
 
 	if (self->current_state == self->last_state) {
-		// g_debug("ccchh-dooris-widget: state unchanged");
+		g_debug("ccchh-dooris-widget: state unchanged");
+		g_static_mutex_unlock(&local_update_mutex);
 		return FALSE;
 	}
 
@@ -327,15 +351,15 @@ update (CCCHHDoorIsDesktopWidget *self)
 	switch (self->current_state) {
 		case DOORIS_OPEN:
 			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_OPEN);
-			// g_debug("ccchh-dooris-widget: image_current set to open");
+			g_debug("ccchh-dooris-widget: image_current set to open");
 			break;
 		case DOORIS_CLOSED:
 			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_CLOSED);
-			// g_debug("ccchh-dooris-widget: image_current set to closed");
+			g_debug("ccchh-dooris-widget: image_current set to closed");
 			break;
 		default:
 			self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
-			// g_debug("ccchh-dooris-widget: image_current set to fnord");
+			g_debug("ccchh-dooris-widget: image_current set to fnord");
 			break;
 	}
 
@@ -344,10 +368,13 @@ update (CCCHHDoorIsDesktopWidget *self)
 	// state is valid and has changed since last known state
 	if (self->current_state != DOORIS_FNORD && self->current_state != self->last_known_state) {
 		self->last_known_state = self->current_state;
-		// g_debug("ccchh-dooris-widget: state changed");
+		g_debug("ccchh-dooris-widget: state changed");
+		g_static_mutex_unlock(&local_update_mutex);
 		return TRUE;
 	}
 
+	g_static_mutex_unlock(&local_update_mutex);
+
 	return FALSE;
 }
 
@@ -357,10 +384,10 @@ draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 {
 //  GtkWidget *widget = GTK_WIDGET(self);
   
-	// g_debug("ccchh-dooris-widget: draw()");
+	g_debug("ccchh-dooris-widget: draw()");
 
 	if (!self->image_current) {
-		// g_debug("ccchh-dooris-widget: draw() without image");
+		g_debug("ccchh-dooris-widget: draw() without image");
 		self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
 	}
 
@@ -372,7 +399,7 @@ draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 
 	// if not connected, overlay icon with "disconnected" image
 	if (!self->connected) {
-		// g_debug("ccchh-dooris-widget: not connected, overlay image");
+		g_debug("ccchh-dooris-widget: not connected, overlay image");
 		cairo_surface_t *image_overlay = cairo_image_surface_create_from_png(IMAGE_OVERLAY_DISCONNECTED);
 		cairo_set_source_surface(cr, image_overlay, 5, 5);
 		cairo_paint_with_alpha(cr, self->current_alpha);
@@ -389,7 +416,7 @@ show_status_banner (CCCHHDoorIsDesktopWidget *self)
 	time_t timenow, timediff;
 	gchar *text, *ts_text;
 
-	// g_debug("ccchh-dooris-widget: show_status_banner()");
+	g_debug("ccchh-dooris-widget: show_status_banner()");
 
 	time (&timenow);
 
@@ -443,7 +470,7 @@ show_status_banner (CCCHHDoorIsDesktopWidget *self)
 static void
 force_redraw (CCCHHDoorIsDesktopWidget *self)
 {
-	// g_debug("ccchh-dooris-widget: force_redraw()");
+	g_debug("ccchh-dooris-widget: force_redraw()");
 
 	/* Invalidate the drawing: */
 	GtkWidget *widget = GTK_WIDGET (self);
@@ -472,26 +499,23 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
   
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_timeout()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_timeout()");
 
-// TODO: glib locking?
-// update() must be called only once at a time
-if (self->updating) {
-	// g_debug("ccchh-dooris-widget: update lock set, not updating");
-	return TRUE;
-}
-self->updating = TRUE;
-
-	// stop event when offline
-	// connection event starts the timer again
+	// stop timer when offline
+	// connection event starts it again
 	if (!self->connected) {
-self->updating = FALSE;
 		return FALSE;
 	}
 
+	// update() must be called only once at a time
+	if (!g_static_mutex_trylock(&(self->update_mutex))) {
+		g_debug("ccchh-dooris-widget: update lock set, not updating");
+		return TRUE;
+	}
+
 	// update internal state
 	if (!update(self)) {
-self->updating = FALSE;
+		g_static_mutex_unlock(&(self->update_mutex));
 		// nothing significant to report, keep the timer running
 		// TODO: this could miss to redraw a change to/from DOORIS_FNORD
 		return TRUE;
@@ -504,7 +528,7 @@ self->updating = FALSE;
 	if (self->notify)
 		show_status_banner(self);
 
-self->updating = FALSE;
+	g_static_mutex_unlock(&(self->update_mutex));
 
 	return TRUE; /* keep running this event */
 }
@@ -516,10 +540,10 @@ ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *eve
 	cairo_t *cr  = gdk_cairo_create (widget->window);
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (widget);
 
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_expose_event()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_expose_event()");
 
 	if (!self->image_current) {
-		// g_debug("ccchh-dooris-widget: expose event without image");
+		g_debug("ccchh-dooris-widget: expose event without image");
 		// no state yet, call update()
 		// flash banner if necessary and enabled
 		if (update(self) && self->notify) {
@@ -550,16 +574,13 @@ ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget,
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (widget);
 
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_button_press_event()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_button_press_event()");
  
-// TODO: glib locking?
-// update() must be called only once at a time
-if (self->updating) {
-	// g_debug("ccchh-dooris-widget: update lock set, not updating");
-	return TRUE;
-}
-
-self->updating = TRUE;
+	// update() must be called only once at a time
+	if (!g_static_mutex_trylock(&(self->update_mutex))) {
+		g_debug("ccchh-dooris-widget: update lock set, not updating");
+		return TRUE;
+	}
 
 	// acknowledge button, make icon transparent
 	// TODO: not really highlighting the button
@@ -572,7 +593,7 @@ self->updating = TRUE;
 	// TODO: this does not work immediately, the call to update()
 	// will still see !connected. maybe wait a bit or use a timer event?
 	if (!self->connected && self->force_connection) {
-		// g_debug("ccchh-dooris-widget: not connected, requesting connection");
+		g_debug("ccchh-dooris-widget: not connected, requesting connection");
 		/* Request connection and check for the result */
 		gboolean success = con_ic_connection_connect(self->connection,
 			CON_IC_CONNECT_FLAG_NONE);
@@ -609,7 +630,7 @@ self->updating = TRUE;
 		start_update_timer(self);
 	}
 
-self->updating = FALSE;
+	g_static_mutex_unlock(&(self->update_mutex));
 
 	return TRUE; /* Prevent further handling of this signal. */
 }
@@ -620,52 +641,93 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
   ConIcConnectionEvent *event, gpointer user_data)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (user_data);
+	const gchar *iap_id = con_ic_event_get_iap_id(CON_IC_EVENT(event));
 
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_connection_event()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_connection_event()");
 
 	// libconic signalled a connection state change
 
 	ConIcConnectionStatus status = con_ic_connection_event_get_status(event);
 
-	if (status == CON_IC_STATUS_CONNECTED) {
+	// Problem: When switching from GPRS to/from WLAN
+	// we receive STATUS_CONNECTED for the new IAP first,
+	// then STATUS_DISCONNECTED for the previous IAP
 
-		// we are online now
-		self->connected = TRUE;
-		// g_debug("ccchh-dooris-widget: new connection state: online");
+	// in STATUS_DISCONNECTING, save disconneting IAP ID and
+	// set self->connected to FALSE
+	// STATUS_CONNECTED sets self->connected to TRUE
+	// if STATUS_DISCONNECTED is called with the same IAP ID
+	// and self->connected has reverted to TRUE, ignore the event
 
-		// start timer
-		start_update_timer(self);
+	switch (status) {
+		case CON_IC_STATUS_DISCONNECTING:
 
-		// force redraw to remove disconnection overlay
-		force_redraw(self);
-	} else if (status == CON_IC_STATUS_DISCONNECTED) {
+			// save disconnecting IAP ID
 
-		// we are offline now
-		self->connected = FALSE;
-		// g_debug("ccchh-dooris-widget: new connection state: offline");
+			if (self->disconnecting) {
+				g_free(self->disconnecting);
+				self->disconnecting = NULL;
+			}
 
-//const gchar *iap_id = con_ic_event_get_iap_id(CON_IC_EVENT(event));
-//const gchar *bearer = con_ic_event_get_bearer_type(CON_IC_EVENT(event));
-//gchar *text = g_strdup_printf("
-// // g_debug("Hey, we are connected to IAP %s with bearer %s!", iap_id, bearer);
-// error = con_ic_connection_event_get_error(event);
-//hildon_banner_show_information_with_markup (GTK_WIDGET (self),
-//	NULL, /* icon_name - ignored */
-//	text);
-//g_free (text);
+			if (iap_id)
+				self->disconnecting = g_strdup(iap_id);
 
-		// don't stop timer here
-		// it will deactivate itsef if !connected
-		// frequent stop/start would prevent it from firing
-//		// stop timer
-//		stop_update_timer(self);
+			self->connected = FALSE;
 
-		// force redraw to add disconnection overlay
-		force_redraw(self);
+			break;
+
+		case CON_IC_STATUS_CONNECTED:
+
+			// we are online now
+
+			if (!self->connected) {
+				self->connected = TRUE;
+				g_debug("ccchh-dooris-widget: new connection state: online");
+
+				// start timer
+				start_update_timer(self);
+
+				// force redraw to remove disconnection overlay
+				force_redraw(self);
+			}
+
+			break;
+
+		case CON_IC_STATUS_DISCONNECTED:
+
+			if (iap_id && self->disconnecting && self->connected &&
+					!strcmp(iap_id, self->disconnecting))
+			{
+
+				g_debug("ccchh-dooris-widget: ignoring offline event from disconnecting IAP");
+
+			} else {
+
+				// we are offline now
+
+				self->connected = FALSE;
+				g_debug("ccchh-dooris-widget: new connection state: offline");
+
+				// don't stop timer here
+				// it will deactivate itsef if !connected
+				// frequent stop/start would prevent it from firing
+
+				// force redraw to add disconnection overlay
+				force_redraw(self);
+			}
+
+			if (self->disconnecting) {
+				g_free(self->disconnecting);
+				self->disconnecting = NULL;
+			}
+
+			break;
+
+		case CON_IC_STATUS_NETWORK_UP:
+
+			// Nothing to do here
+			break;
 	}
-
-	// don't do anything on CON_IC_STATUS_DISCONNECTING
-	// or CON_IC_STATUS_NETWORK_UP
 }
 
 
@@ -679,12 +741,12 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 	g_assert (self->settings_window);
 	g_assert (self->settings_window == GTK_WIDGET (dialog));
   
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_settings_dialog_response()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_settings_dialog_response()");
 
 	// only use new settings if user pressed the OK button
 	// TODO: add reset button to restore defaults
 	if (response_id == GTK_RESPONSE_ACCEPT) {
-		// g_debug("ccchh-dooris-widget: response accept");
+		g_debug("ccchh-dooris-widget: response accept");
 
 		// status URL
 		if (self->url) {
@@ -696,13 +758,13 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 //		self->url = g_strdup(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
 		// try to build a valid URI from user input
 		self->url = gnome_vfs_make_uri_from_input(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
-		// g_debug("ccchh-dooris-widget: new URL: %s\n", self->url);
+		g_debug("ccchh-dooris-widget: new URL: %s\n", self->url);
 
 
 		// update interval
 		const gchar *choice = hildon_button_get_value(HILDON_BUTTON (self->update_picker));
 		self->update = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
-		// g_debug("ccchh-dooris-widget: new update interval: %ld\n", (long)self->update);
+		g_debug("ccchh-dooris-widget: new update interval: %ld\n", (long)self->update);
 
 		// if a timer was running, reset it
 		if (stop_update_timer(self)) {
@@ -712,9 +774,9 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 		// banner notifications
 		self->notify = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->notify_button));
 		if (self->notify) {
-			// g_debug("ccchh-dooris-widget: notification enabled");
+			g_debug("ccchh-dooris-widget: notification enabled");
 		} else {
-			// g_debug("ccchh-dooris-widget: notification disabled");
+			g_debug("ccchh-dooris-widget: notification disabled");
 		}
 
 		// monitor internet connection state, only update if online
@@ -738,9 +800,9 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 		// button forces internet connection
 		self->force_connection = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connforce_button));
 		if (self->force_connection) {
-			// g_debug("ccchh-dooris-widget: button forces connection");
+			g_debug("ccchh-dooris-widget: button forces connection");
 		} else {
-			// g_debug("ccchh-dooris-widget: button does not force connection");
+			g_debug("ccchh-dooris-widget: button does not force connection");
 		}
 	}
 
@@ -773,7 +835,7 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (widget);
  
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_show_settings()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_show_settings()");
 
 	// could this be called when the settings window is already open?
 	if (!self->settings_window) {
@@ -886,7 +948,7 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 static void
 ccchh_dooris_desktop_widget_realize (GtkWidget *widget)
 {
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_realize()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_realize()");
 
 	/* Use An RGBA colormap rather than RGB, 
 	 * so we can use transparency in our expose_event() implementation.
@@ -908,7 +970,7 @@ ccchh_dooris_desktop_widget_dispose (GObject *object)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (object);
  
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_dispose()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_dispose()");
 
 	// remove settings window if it is still open
 	if (self->settings_window) {
@@ -927,7 +989,7 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (object);
 
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_finalize()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_finalize()");
  
 	stop_update_timer(self);
   
@@ -938,9 +1000,8 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 		cairo_surface_destroy(self->image_current);
 	}
 
-	// shut down gnome vfs library
-	// TODO: is this wise? what if other widgets use it?
-//	gnome_vfs_shutdown();
+	// don'shut down gnome vfs library here
+	// other parts of the desktop might still use it
  
 	if (self->url) {
 //		g_free(self->url);
@@ -960,7 +1021,7 @@ ccchh_dooris_desktop_widget_class_init (CCCHHDoorIsDesktopWidgetClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_class_init()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_class_init()");
 
 	object_class->dispose = ccchh_dooris_desktop_widget_dispose;
 	object_class->finalize = ccchh_dooris_desktop_widget_finalize;
@@ -973,14 +1034,14 @@ ccchh_dooris_desktop_widget_class_init (CCCHHDoorIsDesktopWidgetClass *klass)
 static void
 ccchh_dooris_desktop_widget_class_finalize (CCCHHDoorIsDesktopWidgetClass *klass G_GNUC_UNUSED)
 {
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_class_finalize()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_class_finalize()");
 }
 
 
 static void
 ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 {
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_init()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_init()");
 
 	// initialize GnomeVFS library
 	if (!gnome_vfs_init()) {
@@ -1015,8 +1076,9 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	self->image_current = NULL;
 	self->current_alpha = IMAGE_ALPHA_NORMAL;
 
-	self->updating = FALSE;
+	g_static_mutex_init(&(self->update_mutex));
 	self->force_banner = FALSE;
+	self->disconnecting = NULL;
 
 	// set widget size
 	// TODO: is this the right way?
@@ -1052,7 +1114,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 		self->connected = TRUE;
 		start_update_timer(self);
 	}
-	// g_debug("ccchh-dooris-widget: initial URL: %s\n", self->url);
+	g_debug("ccchh-dooris-widget: initial URL: %s\n", self->url);
 }
 
 
@@ -1060,6 +1122,6 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 CCCHHDoorIsDesktopWidget*
 ccchh_dooris_desktop_widget_new (void)
 {
-	// g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_new()");
+	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_new()");
 	return g_object_new (TYPE_CCCHH_DOORIS_DESKTOP_WIDGET, NULL);
 }
